# coding: utf-8

# flake8: noqa
"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    The version of the OpenAPI document: 1.111
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""

# we can not import model classes here because that would create a circular
# reference which would not work in python2
